@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ public:
 #ifdef _ND_USE_PLUGINS
         nd_plugins *plugin_detections,
 #endif
-        nd_devices &devices,
+        nd_device &devices,
         ndDNSHintCache *dhc = NULL,
         ndFlowHashCache *fhc = NULL,
         uint8_t private_addr = 0);
@@ -82,10 +82,9 @@ protected:
     nd_plugins *plugins;
 #endif
     struct ndpi_detection_module_struct *ndpi;
-    uint32_t custom_proto_base;
     nd_private_addr private_addrs;
 
-    nd_devices &devices;
+    nd_device &devices;
 
     ndDNSHintCache *dhc;
     ndFlowHashCache *fhc;
@@ -96,12 +95,16 @@ protected:
     pthread_cond_t pkt_queue_cond;
     pthread_mutex_t pkt_queue_cond_mutex;
 
-    unsigned flows;
-#if 0
-    void DumpFlows(void);
-#endif
-    void ProcessPacketQueue(bool flush);
+    size_t flows;
+
+    ndFlowParser parser;
+
+    void ProcessPacketQueue(void);
     void ProcessPacket(ndDetectionQueueEntry *entry);
+    bool ProcessALPN(ndDetectionQueueEntry *entry, bool client = true);
+
+    void SetDetectedApplication(ndDetectionQueueEntry *entry, nd_app_id_t app_id);
+    void CopyHostname(char *dst, const char *src, size_t length);
 };
 
 typedef map<int16_t, ndDetectionThread *> nd_detection_threads;
