@@ -42,16 +42,12 @@ public:
     void save(void);
 
     size_t size(void) {
-        size_t count = 0;
-        if (pthread_mutex_lock(&lock) == 0) {
-            count = map_ar.size();
-            pthread_mutex_unlock(&lock);
-        }
-        return count;
+        unique_lock<mutex> ul(lock);
+        return map_ar.size();
     };
 
 protected:
-    pthread_mutex_t lock;
+    mutex lock;
     nd_dns_ar map_ar;
 };
 
