@@ -41,51 +41,21 @@ static int krb_decode_asn1_length(struct ndpi_detection_module_struct *ndpi_stru
                                   size_t * const kasn1_offset)
 {
   struct ndpi_packet_struct * const packet = &ndpi_struct->packet;
-  unsigned char length_octet;
-  int length;
+  int64_t length;
+  u_int16_t value_len;
 
-  length_octet = packet->payload[*kasn1_offset];
+  length = ndpi_asn1_ber_decode_length(&packet->payload[*kasn1_offset],
+				       packet->payload_packet_len - *kasn1_offset,
+				       &value_len);
 
-  if (length_octet == 0xFF)
-  {
-    /* Malformed Packet */
-    return -1;
-  }
-
-  if ((length_octet & 0x80) == 0)
-  {
-    /* Definite, short */
-    length = length_octet & 0x7F;
-    (*kasn1_offset)++;
-  } else {
-    /* Definite, long or indefinite (not support by this implementation) */
-    if ((length_octet & 0x7F) == 0)
-    {
-      /* indefinite, unsupported */
-      return -1;
-    }
-
-    length_octet &= 0x7F;
-    if (length_octet > 4 /* We support only 4 additional length octets. */ ||
-        packet->payload_packet_len <= *kasn1_offset + length_octet + 1)
-    {
-      return -1;
-    }
-
-    int i = 1;
-    length = 0;
-    for (; i <= length_octet; ++i)
-    {
-      length |= (unsigned int)packet->payload[*kasn1_offset + i] << (length_octet - i) * 8;
-    }
-    *kasn1_offset += i;
-  }
-
-  if (packet->payload_packet_len < *kasn1_offset + length)
+  if (length == -1 ||
+      packet->payload_packet_len < *kasn1_offset + value_len + length)
   {
     return -1;
   }
 
+  *kasn1_offset += value_len;
+
   return length;
 }
 
@@ -203,11 +173,11 @@ static int krb_decode_asn1_blocks_skip(struct ndpi_detection_module_struct *ndpi
 static void krb_strncpy_lower(char * const dst, size_t dst_siz,
                               char const * const src, size_t src_siz)
 {
-   int dst_len = ndpi_min(src_siz, dst_siz - 1);
+  int i, dst_len = ndpi_min(src_siz, dst_siz - 1);
 
    dst[dst_len] = '\0';
 
-   for(int i = 0; i < dst_len; ++i)
+   for(i = 0; i < dst_len; ++i)
    {
      if (ndpi_isprint(src[i]) == 0)
      {
@@ -424,7 +394,7 @@ void ndpi_search_kerberos(struct ndpi_detection_module_struct *ndpi_struct,
 	
 	return;
       } else if(kerberos_len == expected_len) {
-	if(packet->payload_packet_len > 128) {
+	if(packet->payload_packet_len > 64) {
 	  u_int16_t koffset, i;
 
 	  for(i=8; i<16; i++)
@@ -444,6 +414,7 @@ void ndpi_search_kerberos(struct ndpi_detection_module_struct *ndpi_struct,
 
 	  if(((packet->payload[koffset] == 0x0A)
 	      || (packet->payload[koffset] == 0x0C)
+	      || (packet->payload[koffset] == 0x1E)
 	      || (packet->payload[koffset] == 0x0D)
 	      || (packet->payload[koffset] == 0x0E))) {
 	    u_int16_t koffsetp, body_offset = 0, pad_len;
@@ -671,7 +642,7 @@ void ndpi_search_kerberos(struct ndpi_detection_module_struct *ndpi_struct,
 
 	      if (krb_parse(ndpi_struct, flow, 8) != 0)
 	      {
-	        NDPI_LOG_ERR(ndpi_struct, "[TGS-REP] Invalid packet received\n");
+	        NDPI_LOG_DBG(ndpi_struct, "[TGS-REP] Invalid packet received\n");
 	        return;
 	      }
 	      NDPI_LOG_DBG(ndpi_struct,
@@ -679,6 +650,12 @@ void ndpi_search_kerberos(struct ndpi_detection_module_struct *ndpi_struct,
 	                   sport, dport, flow->protos.kerberos.hostname, flow->protos.kerberos.domain,
 	                   flow->protos.kerberos.username);
 	      flow->extra_packets_func = NULL;
+	    } else if(msg_type == 0x1e) /* Error */ {
+#ifdef KERBEROS_DEBUG
+	      printf("[Kerberos] Processing KRB-Error\n");
+#endif
+	      /* Nothing specific to do; stop dissecting this flow */
+	      flow->extra_packets_func = NULL;
 	    }
 
 	    return;
