@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,10 +30,9 @@
 #include <queue>
 #include <deque>
 #include <sstream>
-#ifdef HAVE_ATOMIC
 #include <atomic>
-#endif
 #include <regex>
+#include <mutex>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -57,6 +56,7 @@ using namespace std;
 
 #include "netifyd.h"
 
+#include "nd-config.h"
 #include "nd-ndpi.h"
 #include "nd-json.h"
 #include "nd-thread.h"
@@ -64,7 +64,7 @@ using namespace std;
 #include "nd-sink.h"
 #include "nd-signal.h"
 
-extern nd_global_config nd_config;
+extern ndGlobalConfig nd_config;
 
 static int nd_curl_debug(CURL *ch __attribute__((unused)),
     curl_infotype type, char *data, size_t size, void *param)
@@ -77,27 +77,27 @@ static int nd_curl_debug(CURL *ch __attribute__((unused)),
     switch (type) {
     case CURLINFO_TEXT:
         buffer.assign(data, size);
-        nd_debug_printf("%s: %s", thread->GetTag().c_str(), buffer.c_str());
+        nd_dprintf("%s: %s", thread->GetTag().c_str(), buffer.c_str());
         break;
     case CURLINFO_HEADER_IN:
         buffer.assign(data, size);
-        nd_debug_printf("%s: <-- %s", thread->GetTag().c_str(), buffer.c_str());
+        nd_dprintf("%s: <-- %s", thread->GetTag().c_str(), buffer.c_str());
         break;
     case CURLINFO_HEADER_OUT:
         buffer.assign(data, size);
-        nd_debug_printf("%s: --> %s", thread->GetTag().c_str(), buffer.c_str());
+        nd_dprintf("%s: --> %s", thread->GetTag().c_str(), buffer.c_str());
         break;
     case CURLINFO_DATA_IN:
-        nd_debug_printf("%s: <-- %lu data bytes\n", thread->GetTag().c_str(), size);
+        nd_dprintf("%s: <-- %lu data bytes\n", thread->GetTag().c_str(), size);
         break;
     case CURLINFO_DATA_OUT:
-        nd_debug_printf("%s: --> %lu data bytes\n", thread->GetTag().c_str(), size);
+        nd_dprintf("%s: --> %lu data bytes\n", thread->GetTag().c_str(), size);
         break;
     case CURLINFO_SSL_DATA_IN:
-        nd_debug_printf("%s: <-- %lu SSL bytes\n", thread->GetTag().c_str(), size);
+        nd_dprintf("%s: <-- %lu SSL bytes\n", thread->GetTag().c_str(), size);
         break;
     case CURLINFO_SSL_DATA_OUT:
-        nd_debug_printf("%s: --> %lu SSL bytes\n", thread->GetTag().c_str(), size);
+        nd_dprintf("%s: --> %lu SSL bytes\n", thread->GetTag().c_str(), size);
         break;
     default:
         break;
@@ -134,8 +134,8 @@ static int ndSinkThread_progress(void *user,
     return 0;
 }
 
-ndSinkThread::ndSinkThread()
-    : ndThread("nd-sink", -1),
+ndSinkThread::ndSinkThread(int16_t cpu)
+    : ndThread("nd-sink", (long)cpu),
     headers(NULL), headers_gz(NULL), pending_size(0), post_errors(0),
     update_imf(1), update_count(0)
 {
@@ -244,9 +244,9 @@ void *ndSinkThread::Entry(void)
 {
     int rc;
 
-    nd_debug_printf("%s: thread started.\n", tag.c_str());
+    nd_dprintf("%s: thread started.\n", tag.c_str());
 
-    while (terminate == false) {
+    while (ShouldTerminate() == false) {
         Lock();
 
         if (uploads.size() == 0) {
@@ -277,15 +277,25 @@ void *ndSinkThread::Entry(void)
             uploads.pop();
 
             while (pending_size > nd_config.max_backlog) {
-                pending_size -= pending.front().second.size();
-                pending.pop_front();
+                size_t overflow = pending.front().second.size();
+                nd_printf("%s: Backlog buffer full (%lu > max_backlog: %lu).\n",
+                    tag.c_str(), pending_size, nd_config.max_backlog);
+
+                if (pending_size - overflow > 0) {
+                    pending_size -= overflow;
+                    pending.pop_front();
+
+                    nd_dprintf("%s: Ejected %lu bytes from backlog buffer.\n",
+                        tag.c_str(), overflow);
+                }
+                else break;
             }
         }
         while (uploads.size() > 0);
 
         Unlock();
 
-        if (terminate == false && pending.size() > 0) Upload();
+        if (ShouldTerminate() == false && pending.size() > 0) Upload();
     }
 
     return NULL;
@@ -297,10 +307,12 @@ void ndSinkThread::Terminate(void)
 
     Lock();
 
-    if ((rc = pthread_cond_broadcast(&uploads_cond)) != 0)
+    if ((rc = pthread_cond_broadcast(&uploads_cond)) != 0) {
+        Unlock();
         throw ndSinkThreadException(strerror(rc));
+    }
 
-    terminate = true;
+    ndThread::Terminate();
 
     Unlock();
 }
@@ -317,8 +329,10 @@ void ndSinkThread::QueuePush(const string &json)
 
     uploads.push(json);
 
-    if ((rc = pthread_cond_broadcast(&uploads_cond)) != 0)
+    if ((rc = pthread_cond_broadcast(&uploads_cond)) != 0) {
+        Unlock();
         throw ndSinkThreadException(strerror(rc));
+    }
 
     Unlock();
 }
@@ -404,17 +418,22 @@ void ndSinkThread::CreateHeaders(void)
     }
 
     string digest;
-    nd_sha1_to_string(nd_config.digest_sink_config, digest);
 
+    nd_sha1_to_string(nd_config.digest_legacy_config, digest);
     ostringstream conf_digest;
     conf_digest << "X-Digest-Sink: " << digest;
 
+    nd_sha1_to_string(nd_config.digest_app_config, digest);
+    ostringstream app_digest;
+    app_digest << "X-Digest-Apps: " << digest;
+
     headers = curl_slist_append(headers, user_agent.str().c_str());
     headers = curl_slist_append(headers, "Content-Type: application/json");
     headers = curl_slist_append(headers, uuid.str().c_str());
     headers = curl_slist_append(headers, uuid_serial.str().c_str());
     headers = curl_slist_append(headers, uuid_site.str().c_str());
     headers = curl_slist_append(headers, conf_digest.str().c_str());
+    headers = curl_slist_append(headers, app_digest.str().c_str());
 
     headers_gz = curl_slist_append(headers_gz, user_agent.str().c_str());
     headers_gz = curl_slist_append(headers_gz, "Content-Type: application/json");
@@ -423,6 +442,7 @@ void ndSinkThread::CreateHeaders(void)
     headers_gz = curl_slist_append(headers_gz, uuid_serial.str().c_str());
     headers_gz = curl_slist_append(headers_gz, uuid_site.str().c_str());
     headers_gz = curl_slist_append(headers_gz, conf_digest.str().c_str());
+    headers_gz = curl_slist_append(headers_gz, app_digest.str().c_str());
 
     for (map<string, string>::const_iterator i = nd_config.custom_headers.begin();
         i != nd_config.custom_headers.end(); i++) {
@@ -454,7 +474,7 @@ void ndSinkThread::Upload(void)
 
     if (post_errors == nd_config.sink_max_post_errors) {
         free(nd_config.url_sink);
-        nd_config.url_sink = strdup(ND_URL_SINK);
+        nd_config.url_sink = strdup(nd_config.url_sink_provision);
         nd_printf("%s: reverted to default sink URL: %s\n", tag.c_str(),
             nd_config.url_sink);
 
@@ -469,13 +489,13 @@ void ndSinkThread::Upload(void)
     if (++update_count == update_imf || post_errors > 0)
         update_count = 0;
     else {
-        nd_debug_printf("%s: payload upload delay: %u of %u\n",
+        nd_dprintf("%s: payload upload delay: %u of %u\n",
             tag.c_str(), update_count, update_imf);
         return;
     }
 
     do {
-        nd_debug_printf("%s: payload %lu/%lu (%d of %d bytes)...\n",
+        nd_dprintf("%s: payload %lu/%lu (%d of %d bytes)...\n",
             tag.c_str(), ++xfer, total, pending.front().second.size(), pending_size);
 
         if (! pending.front().first)
@@ -529,7 +549,7 @@ void ndSinkThread::Upload(void)
         curl_easy_getinfo(ch, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &content_length);
 
         if (content_type == NULL) {
-            nd_debug_printf("%s: Missing content type.\n", tag.c_str());
+            nd_dprintf("%s: Missing content type.\n", tag.c_str());
 
             ndJsonResponse *response = new ndJsonResponse(
                 ndJSON_RESP_INVALID_CONTENT_TYPE,
@@ -542,7 +562,7 @@ void ndSinkThread::Upload(void)
                 PushResponse(response);
         }
         else if (content_length == 0.0f) {
-            nd_debug_printf("%s: Zero-length content length.\n", tag.c_str());
+            nd_dprintf("%s: Zero-length content length.\n", tag.c_str());
 
             ndJsonResponse *response = new ndJsonResponse(
                 ndJSON_RESP_INVALID_RESPONSE,
@@ -556,7 +576,7 @@ void ndSinkThread::Upload(void)
         }
         else if (strcasecmp("application/json", content_type) != 0) {
 
-            nd_debug_printf("%s: Unexpected content type.\n", tag.c_str());
+            nd_dprintf("%s: Unexpected content type.\n", tag.c_str());
 
             ndJsonResponse *response = new ndJsonResponse(
                 ndJSON_RESP_INVALID_CONTENT_TYPE,
@@ -585,7 +605,7 @@ void ndSinkThread::Upload(void)
                     fwrite(pending.front().second.data(),
                         1, pending.front().second.size(), hf);
                     fclose(hf);
-                    nd_debug_printf(
+                    nd_dprintf(
                         "%s: wrote rejected payload to: %s\n",
                         tag.c_str(), ND_JSON_FILE_BAD_SEND);
                 }
@@ -603,7 +623,7 @@ void ndSinkThread::Upload(void)
         pending_size -= pending.front().second.size();
         pending.pop_front();
 
-        if (pending.size() == 0 || terminate)
+        if (pending.size() == 0 || ShouldTerminate())
             flush_queue = false;
         else {
             Lock();
@@ -655,7 +675,7 @@ string ndSinkThread::Deflate(const string &data)
 
     if (ND_DEBUG || ND_DEBUG_UPLOAD) {
 
-        nd_debug_printf(
+        nd_dprintf(
             "%s: payload compressed: %lu -> %lu: %.1f%%\n",
             tag.c_str(), data.size(), buffer.size(),
             100.0f - ((float)buffer.size() * 100.0f / (float)data.size())
@@ -707,11 +727,20 @@ void ndSinkThread::ProcessResponse(void)
             for (ndJsonData::const_iterator i = response->data.begin();
                 i != response->data.end(); i++) {
 
-                if (i->first == ND_CONF_SINK_BASE) {
+                if (i->first == ND_CONF_APP_BASE) {
 
-                    if (nd_save_response_data(ND_CONF_SINK_PATH, i->second) == 0 &&
+                    if (nd_save_response_data(ND_CONF_APP_PATH, i->second) == 0 &&
                         nd_sha1_file(
-                            nd_config.path_sink_config, nd_config.digest_sink_config
+                            nd_config.path_app_config, nd_config.digest_app_config
+                        ) == 0)
+                        create_headers = true;
+                }
+
+                if (i->first == ND_CONF_LEGACY_BASE) {
+
+                    if (nd_save_response_data(ND_CONF_LEGACY_PATH, i->second) == 0 &&
+                        nd_sha1_file(
+                            nd_config.path_legacy_config, nd_config.digest_legacy_config
                         ) == 0)
                         create_headers = true;
                 }
@@ -721,7 +750,7 @@ void ndSinkThread::ProcessResponse(void)
         }
 
         if (response->update_imf > 0 && response->update_imf != update_imf) {
-            nd_debug_printf("%s: changing update multiplier from: %u"
+            nd_dprintf("%s: changing update multiplier from: %u"
                 " to: %u\n", tag.c_str(), update_imf, response->update_imf);
             update_imf = response->update_imf;
         }
@@ -745,7 +774,7 @@ void ndSinkThread::ProcessResponse(void)
     }
 
     if (response != NULL) {
-        nd_debug_printf("%s: [%d] %s\n", tag.c_str(),
+        nd_dprintf("%s: [%d] %s\n", tag.c_str(),
             response->resp_code,
             (response->resp_message.size() > 0) ?
                 response->resp_message.c_str() : "(no message)");
